/* boot.S - bootstrap the kernel */
/* first we give some definitions, to make our code more readable later */

#define ASM     1

#define MULTIBOOT_HEADER_MAGIC          0x1BADB002 /* magic no. for multiboot header */
#define MULTIBOOT_HEADER_FLAGS          0x00000003 /* flags for multiboot header */
#define STACK_SIZE 0x4000                          /* size of our stack (16KB) */

/* On some systems we have to jump from assembly to C by referring to
   the C name prefixed by an underscore. This is all defined during
   configuration. We do not worry about it here and make sure we can
   handle either (HAVE_ASM_USCORE is defined by configure). */

#ifdef HAVE_ASM_USCORE
# define EXT_C(sym)                     _ ## sym
#else
# define EXT_C(sym)                     sym
#endif

	/* The text segment will contain code, the data segment data
	    and the bss segment zero-filled static variables */

	.text
	.globl  start /* start is the global entry point into kernel */
start:

	/* As we need the multiboot magic values in the beginning of
	our file, we will add them presently. The real code
	continues right after those 3 long words, so jump over them
	*/
	jmp     multiboot_entry

	/* Align the multiboot header at a 32 bits boundary.  */
	.align 4

multiboot_header:	 /* Now comes the multiboot header.  */
	.long   MULTIBOOT_HEADER_MAGIC
	.long   MULTIBOOT_HEADER_FLAGS
	.long   -(MULTIBOOT_HEADER_MAGIC + MULTIBOOT_HEADER_FLAGS)

multiboot_entry:
	/* We do not like assembly. All we want is to create a
	a stack and start executing C code */

	/* Initialize the stack pointer (definition of stack follows) */
	movl    $(stack + STACK_SIZE), %esp

	/* Now enter the C main function (EXT_C will handle the underscore, if needed)  */
	call    EXT_C(cmain)

	        /* Halt.  When we return from C, we end up here. */
loop:
	hlt
	jmp     loop

	.section ".bss"
	        /* We define our stack area. The .comm pseudo op declares
	           a common symbol (common means that if a symbol with the
	           same name is defined in another object file, it can be
	           they can be merged (after all, we only need one stack).
	           If the loader (ld) does not find an existing definition,
	           it will allocate STACK_SIZE bytes of uninitialised
	           memory. */
	.comm   stack, STACK_SIZE

